﻿using Aop.Api.Domain;
using Common;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using System.Web.Http;
using TGood.Xian.LogManager;
using WxPayAPI.Models;
using wxWebAPI.Common;
using wxWebAPI.Models;
using wxWebAPI.Models.cardapplet;

namespace WxPayAPI
{
    public partial class OrdersController
    {
        #region 微信服务商支付相关

        [HttpPost, HttpOptions]
        public JObject LoginFuwushang([FromBody]LoginInfo loginInfo)
        {
            JObject result = new JObject();
            try
            {
                var payLogic = new PayLogic();
                string userToken = payLogic.Login(loginInfo);
                if (!string.IsNullOrEmpty(userToken))
                {
                    result.Add("code", 1);
                    result.Add("data", userToken);
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error("PayController - Login", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        [HttpPost, HttpOptions]
        public JObject NewWXMINAPayFuwushang([FromBody]WXMINAPayParameter param)
        {
            JObject result = new JObject();
            try
            {
                var payLogic = new PayLogic();
                string parameters = payLogic.NewWXMINAPayFuwushang(param);
                if (!string.IsNullOrEmpty(parameters) && parameters != "子商户信息不存在")
                {
                    result.Add("code", 1);
                    result.Add("data", parameters);
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", parameters);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"PayController - WXMINAPayFuwushang - param=[{param.ToString()}]", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        [HttpPost, HttpOptions]
        public JObject WXMINAPayFuwushang([FromBody]WXMINAPayParameter param)
        {
            JObject result = new JObject();
            try
            {
                var payLogic = new PayLogic();
                string parameters = payLogic.WXMINAPayFuwushang(param);
                if (!string.IsNullOrEmpty(parameters))
                {
                    result.Add("code", 1);
                    result.Add("data", parameters);
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"PayController - WXMINAPayFuwushang - param=[{param.ToString()}]", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        [HttpPost, HttpOptions]
        public System.Net.Http.HttpResponseMessage WXMINAPayNotifyFuwushang()
        {
            LogEvent.WebAPILoger.Debug("WXMINAPayNotifyFuwushang, WXMINAPayNotify进来了");
            string result = string.Empty;
            try
            {
                var payLogic = new PayLogic();
                result = payLogic.WXMINAPayNotifyFuwushang(WxPayConfig.KEY);
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error("PayController - WXMINAPayNotifyFuwushang", ex);
            }
            var response = new System.Net.Http.HttpResponseMessage();
            response.StatusCode = System.Net.HttpStatusCode.OK;
            response.Content = new System.Net.Http.StringContent(result);
            return response;
        }

        /// <summary>
        /// 智电云联回调
        /// </summary>
        /// <returns></returns>
        [HttpPost, HttpOptions]
        public System.Net.Http.HttpResponseMessage ZDYLWXMINAPayNotifyFuwushang()
        {
            string result = string.Empty;
            try
            {
                var payLogic = new PayLogic();
                result = payLogic.WXMINAPayNotifyFuwushang(WxPayConfig.ZDYLKEY);
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error("PayController - WXMINAPayNotifyFuwushang", ex);
            }
            var response = new System.Net.Http.HttpResponseMessage();
            response.StatusCode = System.Net.HttpStatusCode.OK;
            response.Content = new System.Net.Http.StringContent(result);
            return response;
        }

        /// <summary>
        /// 申请退款
        /// </summary>
        /// <param name="refundParameter"></param>
        /// <returns></returns>
        [HttpPost, HttpOptions]
        public void RefundFuwushang([FromBody] WXRefund refundParameter)
        {
            LogEvent.WebAPILoger.Debug("RefundFuwushang 进来了");
            string result = string.Empty;
            try
            {
                var payLogic = new PayLogic();
                payLogic.RefundFuwushang(refundParameter);

                // result =  payLogic.Refund(refundParameter);
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"PayController - RefundFuwushang.refundParameter={refundParameter.ToString()}", ex);
            }
        }

        /// <summary>
        /// 获取服务发送过来的退款信息，执行退款
        /// </summary>
        /// <param name="refundParameter"></param>
        /// <returns></returns>
        [HttpPost, HttpOptions]
        public string RefundServer([FromBody]WXRefund refundParameter)
        {
            string result = string.Empty;
            try
            {
                var payLogic = new PayLogic();
                result = payLogic.Refund(refundParameter);

            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"PayController - RefundServer.refundParameter={refundParameter.ToString()}", ex);
            }
            return result;
        }

        [HttpPost, HttpOptions]
        public System.Net.Http.HttpResponseMessage WXMINAPayNotifyTest()
        {
            string result = string.Empty;
            try
            {
                var payLogic = new PayLogic();
                result = payLogic.WXMINAPayNotifyTest();
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error("PayController - WXMINAPayNotifyTest", ex);
            }
            var response = new System.Net.Http.HttpResponseMessage();
            response.StatusCode = System.Net.HttpStatusCode.OK;
            response.Content = new System.Net.Http.StringContent(result);
            return response;
            //return result;
        }

        [HttpPost, HttpOptions]
        public string RefundQuery([FromBody]WXRefund refundQuery)
        {
            string apiResultConvert = string.Empty;
            try
            {
                PayLogic payLogic = new PayLogic();

                var result = payLogic.RefundQuery(refundQuery).DicToJson();
                apiResultConvert = new Regex(@"\\u([0-9A-F]{4})", RegexOptions.IgnoreCase | RegexOptions.Compiled)
            .Replace(result, x => string.Empty + Convert.ToChar(Convert.ToUInt16(x.Result("$1"), 16)));
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"PayController - RefundFuwushang.refundParameter={refundQuery.ToString()}", ex);
            }
            return apiResultConvert;
        }

        [HttpGet, HttpOptions]
        public JObject GetTotalFee(string devicecode=null)
        {
            JObject result = new JObject();
            //默认小程序为领充 支付方式为微信
            EnumAppName app = EnumAppName.领充;
            //EnumPayChannel payChannel = EnumPayChannel.微信;
            PayLogic paylogic = new PayLogic();
            ChargeLogic chargeLogic = new ChargeLogic();
            try
            {
                var totalfee = paylogic.GetTotalFee();
                if (totalfee != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(totalfee));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", "fail");
                }
            }
            catch (Exception e)
            {
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("data", "fail");
                LogEvent.WebAPILoger.Error("查询数据错误！", e);
            }
            if (devicecode!=null)
            {
                //(int)chargeLogic.GetPaymentMethod(stationid, ref payChannel, ref app)
                //string stationid = WechatUtility.GetStationId(devicecode);
                result.Add("paymentWay",2 );
                result.Add("app", (int)app);
            }
            return result;
        }

        #endregion

        #region 测试函数

        [HttpGet, HttpOptions]
        public string GetTest(string test)
        {
            //var payLogic = new PayLogic();
            //payLogic.test();


            return test;
        }

        public string GetCertficates(string mch_id)
        {
            string result = string.Empty;
            try
            {
                PayLogic payLogic = new PayLogic();

                result = payLogic.GetCertficates(mch_id);
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"PayController - GetCertficates.mch_id={mch_id}", ex);
            }
            return result;
        }

        #endregion

        #region 微信——卡绑定
        [HttpGet, HttpOptions]
        public JObject GetCardInfoByCompanyID(string userToken, string deviceCode)
        {
            JObject result = new JObject();
            PayLogic pl = new PayLogic();
            try
            {
                IEnumerable<wxWebAPI.Models.cardapplet.CardInfo> cardInfo = pl.GetCardInfoByApplet(userToken, deviceCode);
                if (cardInfo != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(cardInfo));
                }
                else
                {
                    result.Add("code", 1);
                    result.Add("data", "用户此场站无对应卡号！");
                }
            }
            catch (Exception e)
            {
                result.Add("code", 0);
                result.Add("error", $"{e.ToString()}");
            }
            return result;
        }

        [HttpPost, HttpOptions]
        public JObject ApplyCard([FromBody]CardOfApply applycard)
        {
            JObject result = new JObject();
            PayLogic pl = new PayLogic();
            try
            {
                bool apply = pl.ApplyCard(applycard);
            }
            catch (Exception e)
            {
                result.Add("code", 0);
                result.Add("data", e.ToString());
            }
            return result;
        }

        [HttpPost, HttpOptions]
        public JObject CardPayFuwushang([FromBody]cardPayParam param)
        {
            JObject result = new JObject();
            try
            {
                var payLogic = new PayLogic();
                string parameters = payLogic.CardPayFuwushang(param);
                if (!string.IsNullOrEmpty(parameters))
                {
                    result.Add("code", 1);
                    result.Add("data", parameters);
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"PayController - WXMINAPayFuwushang - param=[{param.ToString()}]", ex);
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("error", ex.ToString());

            }
            return result;
        }

        [HttpPost, HttpOptions]
        public System.Net.Http.HttpResponseMessage WXCardPayNotifyFuwushang()
        {
            //LogEvent.WebAPILoger.Debug("WXMINAPayNotifyFuwushang, WXMINAPayNotify进来了");
            string result = string.Empty;
            try
            {
                var payLogic = new PayLogic();
                result = payLogic.WXCardPayNotifyFuwushang();
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error("PayController - WXMINAPayNotifyFuwushang", ex);
            }
            var response = new System.Net.Http.HttpResponseMessage();
            response.StatusCode = System.Net.HttpStatusCode.OK;
            response.Content = new System.Net.Http.StringContent(result);
            return response;
        }

        [HttpPost, HttpOptions]
        public JObject CardStartCharge([FromBody]CardCharge cardCharge)
        {
            JObject result = new JObject();
            PayLogic pl = new PayLogic();
            try
            {
                if (string.IsNullOrEmpty(WechatUtility.GetOpenIDByUserToken(cardCharge.userToken)))
                {
                    result.Add("code", -10);
                    result.Add("error", "Token过期，请重新登陆");
                    return result;
                }
                string success = pl.CardStartCharge(cardCharge);
                switch (success)
                {
                    case "-1":
                        result.Add("code", 0);
                        result.Add("data", "卡余额不足");
                        break;
                    case "0":
                        result.Add("code", 1);
                        result.Add("data", "success");
                        break;
                    case "1":
                        result.Add("code", 0);
                        result.Add("data", "此桩正在充电");
                        break;
                    case "2":
                        result.Add("code", 0);
                        result.Add("data", "启动出错");
                        break;
                    case "3":
                        result.Add("code", 0);
                        result.Add("data", "此桩不能获取正在生效的电费信息");
                        break;
                    default:
                        result.Add("code", 0);
                        result.Add("data", "启动出错");
                        break;
                }
            }
            catch (Exception e)
            {
                result.RemoveAll();
                result.Add("code", 0);
                result.Add("data", e.ToString());
            }
            return result;
        }
        /// <summary>
        /// 获取用户所有卡
        /// </summary>
        [HttpGet, HttpOptions]
        public JObject GetAllCardInfo(string userToken)
        {
            JObject result = new JObject();
            try
            {
                if (string.IsNullOrEmpty(WechatUtility.GetOpenIDByUserToken(userToken)))
                {
                    result.Add("code", -10);
                    result.Add("error", "Token过期，请重新登陆");
                    return result;
                }
                PayLogic pl = new PayLogic();
                IEnumerable<wxWebAPI.Models.cardapplet.CardInfo> cards = pl.GetCardInfo(userToken);
                if (cards != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(cards));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", "查无数据");
                }
            }
            catch (Exception e)
            {
                result.Add("code", 0);
                result.Add("data", $"获取用户卡信息失败 GetAllCardInfo usertoken = {userToken}+{e.ToString()}");
                result.Add("error", e.ToString());
            }
            return result;
        }
        /// <summary>
        /// 获取卡详情
        /// </summary>
        /// <param name="cardid"></param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject GetCardDetails(string cardid)
        {
            JObject result = new JObject();
            PayLogic paylogic = new PayLogic();
            try
            {
                var card = paylogic.GetCardDetails(cardid);
                if (card!=null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(card));
                }
            }
            catch (Exception e)
            {
                result.Add("code", 0);
                result.Add("data", $"失败 cardid={cardid}+{e.ToString()}");
            }
            return result;
        }

        [HttpGet, HttpOptions]
        public JObject GetOrderByCardID(string cardid, int MinID = 0)
        {
            JObject result = new JObject();
            PayLogic paylogic = new PayLogic();
            try
            {
                IEnumerable<Order> orders = paylogic.GetOrderByCardID(cardid, MinID);
                result.Add("code", 1);
                result.Add("data", JProperty.FromObject(orders));
                result.Add("minid", orders.Any() ? orders.Last().id : 0);
            }
            catch (Exception e)
            {
                result.Add("code", 0);
                result.Add("data", $"失败 cardid={cardid}+{e.ToString()}");
            }

            return result;
        }
        /// <summary>
        /// 获取当前会员卡对应的场站
        /// </summary>
        /// <param name="cardid"></param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject GetUsableStation(string cardid)
        {
            JObject result = new JObject();
            PayLogic paylogic = new PayLogic();
            try
            {
                List<LCO_Station> stations = paylogic.GetStationByCardID(cardid);
                result.Add("code", 1);
                result.Add("data", JProperty.FromObject(stations));
            }
            catch (Exception e)
            {
                result.Add("code", 0);
                result.Add("data", $"失败GetStation  cardid={cardid},{e.ToString()}");
            }
            return result;
        }
        [HttpGet, HttpOptions]
        public JObject GetUserBaseInfo(string usertoken)
        {
            JObject result = new JObject();
            try
            {
                if (string.IsNullOrEmpty(WechatUtility.GetOpenIDByUserToken(usertoken)))
                {
                    result.Add("code", -10);
                    result.Add("error", "Token过期，请重新登陆");
                    return result;
                }
                PayLogic paylogic = new PayLogic();
                var baseInfo = paylogic.GetUserBaseInfo(usertoken);
                result.Add("code", 1);
                result.Add("data", JProperty.FromObject(baseInfo));
            }
            catch (Exception e)
            {
                result.Add("code", 0);
                result.Add("data", e.ToString());
            }
            return result;
        }
        #endregion

        #region i电启动接口

        /// <summary>
        /// i电启动接口
        /// </summary>
        /// <returns></returns>
        [HttpPost, HttpOptions]
        public return_start_charge IDianStartDevice([FromBody]return_start_charge request)
        {
            return_start_charge result = new return_start_charge();
            try
            {
                if (request == null)
                {
                    LogEvent.GetLoger("iDian").Error($"PayController - IDianStartDevice Error. request为空");
                    result.ConnectorID = "";
                    result.StartChargeSeq = "";
                    result.StartChargeSeqStat = 0;
                    result.SuccStat = (int)EnumSuccStat.失败;
                    result.FailReason = EnumStartFailReason.其他;

                    return result;
                }

                var payLogic = new PayLogic();
                result = payLogic.IDianStartDevice(request.ConnectorID, request.StartChargeSeq);
            }
            catch (Exception ex)
            {
                LogEvent.GetLoger("iDian").Error($"PayController - IDianStartDevice Error.deviceCode={request.ConnectorID},orderID={request.StartChargeSeq}", ex);
            }
            //var response = new System.Net.Http.HttpResponseMessage();
            //response.StatusCode = System.Net.HttpStatusCode.OK;
            //response.Content = new System.Net.Http.StringContent(result);
            //return response;
            return result;
        }

        #endregion
    }
}
